languages = {
    'python': {
        # see https://regex101.com/r/1ZiRBM/1
        'search_pattern_start': r'^(?!def)(.*)(^|\)|\s|=|\(|\.|!|@)',
        'search_pattern_end': r'(\s?)\(',
        'function_definition_pattern': r'def\s(\w+)\(',
        'magic_method_format': r'^__.*__$',
        'extension': '.py'
    },
    'php': {
        # see https://regex101.com/r/7YVqUM/1
        'search_pattern_start': r'^(?!function)(.*)(->|::|^|\)|\s|=|\(|\.|!)',
        'search_pattern_end': r'(\s?)\(',
        'function_definition_pattern': r'function\s+([^\s\(]+)\s*\(',
        'magic_method_format': r'^__.*$',
        'extension': '.php'
    },
}


def get_extension(language):
    """ Return the extension of a language """

    try:
        return languages[language]['extension']
    except KeyError:
        raise NotImplementedError


def get_function_definition_pattern(language):
    """ Return functions definition pattern """

    try:
        return languages[language]['function_definition_pattern']
    except KeyError:
        raise NotImplementedError


def get_magic_method_format(language):
    """ Return magic method format """

    try:
        return languages[language]['magic_method_format']
    except KeyError:
        raise NotImplementedError


def get_search_pattern(language):
    """ Return search pattern """

    try:
        return (languages[language]['search_pattern_start'], languages[language]['search_pattern_end'])
    except KeyError:
        raise NotImplementedError



import glob
import re
import os

from .supported_languages import get_extension


def parse(path, language='python', exclude=None):
    """ Return a recursive list of files within a directory """

    # Add optional /
    if not path.endswith('/'):
        path += '/'

    # Resolve home path
    path = resolve_home_path(path)

    # Check if dir exists
    if not os.path.isdir(path):
        raise FileNotFoundError

    # absolute path to search all text files inside a specific folder
    path = path + r'**/*' + get_extension(language)
    files = glob.glob(path, recursive=True)

    exclusions = (exclude).split(',') if exclude else []

    result = []
    for file in files:
        # Skip directories with extensions
        if not os.path.isfile(file):
            continue

        # Skip excluded files
        if is_excluded(file, exclusions):
            continue

        result.append(file)

    return result


def resolve_home_path(path):
    """ Resolve home directory """

    if path.startswith('~'):
        path = os.path.expanduser(path)

    return path


def is_excluded(file, exclusions):
    """ Check if file contains an excluded pattern """

    for exclusion in exclusions:
        if exclusion and exclusion.strip() in file:
            return True

    return False



import re

from .supported_languages import get_search_pattern


def search_function_in_file(function_name, file_path, language='python'):
    """ Return a list of functions within a file """

    # Read file
    with open(file_path, 'r') as file:
        try:
            file_content = file.read()
        except UnicodeDecodeError:
            return None

    start, end = get_search_pattern(language)

    # Find all functions
    found = re.search(start + function_name + end, file_content, re.MULTILINE)

    return True if found else False


def iterate_and_search(function_name, files, language):
    """ Return a list of functions within a directory """

    for file_path in files:
        found = search_function_in_file(function_name, file_path, language)

        if found:
            return True

    return False




import re

from .directory_parser import parse
from .supported_languages import get_function_definition_pattern, get_magic_method_format


def find_functions(file_path, language='python'):
    """ Return a list of functions within a file """

    # Read file
    with open(file_path, 'r') as file:
        try:
            file_content = file.read()
        except UnicodeDecodeError:
            return []

    # Find all functions
    functions = re.findall(
        get_function_definition_pattern(language), file_content)

    # Remove magic methods
    functions = remove_magic_methods(functions, language)

    return list(set(functions))


def find_all_functions(files, language='python'):
    """ Return a list of functions within a directory """

    functions_by_file = {}
    for file_path in files:
        functions = find_functions(file_path, language)

        if functions:
            functions_by_file[file_path] = functions

    return functions_by_file


def remove_magic_methods(functions, language='python'):
    """ Remove magic methods from a list of functions """

    magic_method_format = get_magic_method_format(language)

    # Remove magic methods
    result = []
    for function in functions:

        if not re.match(magic_method_format, function):
            result.append(function)

    return result




import argparse

from .modules.directory_parser import parse
from .modules.function_finder import find_all_functions
from .modules.function_search import iterate_and_search


def main():
    """ Return a list of functions within a directory """

    # Parse arguments
    parser = argparse.ArgumentParser()
    parser.add_argument("-l", "--language", type=str, help="Programming language",
                        choices=['python', 'php'], default='python')
    parser.add_argument(
        "-p", "--path", help="Path to directory", required=True)
    parser.add_argument(
        "-x", "--exclude", help="Exclude a file or directory pattern, (comma separated")
    args = parser.parse_args()

    # List filenames
    all_files = [x for x in parse(args.path, args.language, args.exclude)]

    # Search all functions by filenames
    functions_by_file = find_all_functions(all_files, args.language)
    functions_count = sum(len(v) for v in functions_by_file.values())
    print(' * Found {} functions in {} files.'.format("{:,.0f}".format(functions_count),
                                                      "{:,.0f}".format(len(functions_by_file))))

    # Search all functions in all files
    count = 0
    for file_path in functions_by_file:
        # All functions in file
        functions = functions_by_file[file_path]

        # Loop through all functions
        for function in functions:
            res = iterate_and_search(function, all_files, args.language)

            if not res:
                count += 1
                print(
                    '   -> Function `{}` from `{}` not found in any file.'.format(function, file_path))

    print(' * Found {} dead functions.'.format("{:,.0f}".format(count)))

    return None


if __name__ == '__main__':
    main()



from setuptools import setup

import pypandoc

setup(
    name='dead_function_finder',
    version='1.0',
    description='Utility to find dead functions within a codebase.',
    long_description=pypandoc.convert_file('README.md', 'rst'),
    author='Gabriel Bordeaux',
    author_email='pypi@gab.lc',
    url='https://github.com/gabfl/dead_function_finder',
    license='MIT',
    packages=['dead_function_finder', 'dead_function_finder.modules'],
    package_dir={'dead_function_finder': 'src'},
    install_requires=[],  # external dependencies
    entry_points={
        'console_scripts': [
            'dead_function_finder = dead_function_finder.dead_function_finder:main',
        ],
    },
    classifiers=[  # see https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Topic :: Database',
        'Topic :: Database :: Database Engines/Servers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS',
        'Operating System :: POSIX :: Linux',
        'Natural Language :: English',
        #  'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python',
        'Development Status :: 4 - Beta',
        #  'Development Status :: 5 - Production/Stable',
    ],
)
